@@ -2,7 +2,7 @@
 #
 # E-Mail post-processing script for NZBGet
 #
-# Copyright (C) 2013-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+# Copyright (C) 2013-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,6 +60,9 @@
 # SMTP server password, if required.
 #Password=mypass
 
+# To check connection parameters click the button.
+#ConnectionTest@Send Test E-Mail
+
 # Append statistics to the message (yes, no).
 #Statistics=yes
 
@@ -88,11 +91,15 @@ import os
 import sys
 import datetime
 import smtplib
+
 from email.mime.text import MIMEText
-try:
-	from xmlrpclib import ServerProxy # python 2
-except ImportError:
-	from xmlrpc.client import ServerProxy # python 3
+from email.header import Header
+try: # python 2
+	from urllib2 import quote
+	from xmlrpclib import ServerProxy
+except ImportError: # python 3
+	from urllib.parse import quote
+	from xmlrpc.client import ServerProxy
 
 # Exit codes used by NZBGet
 POSTPROCESS_SUCCESS=93
@@ -114,38 +121,46 @@ for	optname in required_options:
 		print('[ERROR] Option %s is missing in configuration file. Please check script settings' % optname[6:])
 		sys.exit(POSTPROCESS_ERROR)
 
-status = os.environ['NZBPP_STATUS']
-total_status = os.environ['NZBPP_TOTALSTATUS']
+# Check if the script is executed from settings page with a custom command
+command = os.environ.get('NZBCP_COMMAND')
+test_mode = command == 'ConnectionTest'
+if command != None and not test_mode:
+	print('[ERROR] Invalid command ' + command)
+	sys.exit(POSTPROCESS_ERROR)
+
+status = os.environ.get('NZBPP_STATUS') if not test_mode else 'SUCCESS/ALL'
+total_status = os.environ.get('NZBPP_TOTALSTATUS') if not test_mode else 'SUCCESS'
 
 # If any script fails the status of the item in the history is "WARNING/SCRIPT".
 # This status however is not passed to pp-scripts in the env var "NZBPP_STATUS"
 # because most scripts are independent of each other and should work even
 # if a previous script has failed. But not in the case of E-Mail script,
 # which should take the status of the previous scripts into account as well.
-if total_status == 'SUCCESS' and os.environ['NZBPP_SCRIPTSTATUS'] == 'FAILURE':
+if total_status == 'SUCCESS' and os.environ.get('NZBPP_SCRIPTSTATUS') == 'FAILURE':
 	total_status = 'WARNING'
 	status = 'WARNING/SCRIPT'
 
 success = total_status == 'SUCCESS'
 
-if success and os.environ.get('NZBPO_SENDMAIL') == 'OnFailure':
+if success and os.environ.get('NZBPO_SENDMAIL') == 'OnFailure' and not test_mode:
 	print('[INFO] Skipping sending of message for successful download')
 	sys.exit(POSTPROCESS_NONE)
 
 if success:
-	subject = 'Success for "%s"' % (os.environ['NZBPP_NZBNAME'])
-	text = 'Download of "%s" has successfully completed.' % (os.environ['NZBPP_NZBNAME'])
+	subject = 'Success for "%s"' % (os.environ.get('NZBPP_NZBNAME', 'Test download'))
+	text = 'Download of "%s" has successfully completed.' % (os.environ.get('NZBPP_NZBNAME', 'Test download'))
 else:
 	subject = 'Failure for "%s"' % (os.environ['NZBPP_NZBNAME'])
 	text = 'Download of "%s" has failed.' % (os.environ['NZBPP_NZBNAME'])
 
 text += '\nStatus: %s' % status
 
-if os.environ.get('NZBPO_STATISTICS') == 'yes' or \
+if (os.environ.get('NZBPO_STATISTICS') == 'yes' or \
 	os.environ.get('NZBPO_NZBLOG') == 'Always' or \
-	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success):
+	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success)) and \
+	not test_mode:
 	# To get statistics or the post-processing log we connect to NZBGet via XML-RPC.
-	# For more info visit http://nzbget.net/RPC_API_reference
+	# For more info visit http://nzbget.net/api
 	# First we need to know connection info: host, port and password of NZBGet server.
 	# NZBGet passes all configuration options to post-processing script as
 	# environment variables.
@@ -156,13 +171,13 @@ if os.environ.get('NZBPO_STATISTICS') == 'yes' or \
 
 	if host == '0.0.0.0': host = '127.0.0.1'
 
-	# Build an URL for XML-RPC requests
-	rpcUrl = 'http://%s:%s@%s:%s/xmlrpc' % (username, password, host, port);
+	# Build a URL for XML-RPC requests
+	rpcUrl = 'http://%s:%s@%s:%s/xmlrpc' % (quote(username), quote(password), host, port);
 
 	# Create remote server object
 	server = ServerProxy(rpcUrl)
 
-if os.environ.get('NZBPO_STATISTICS') == 'yes':
+if os.environ.get('NZBPO_STATISTICS') == 'yes' and not test_mode:
 	# Find correct nzb in method listgroups
 	groups = server.listgroups(0)
 	nzbID = int(os.environ['NZBPP_NZBID'])
@@ -206,7 +221,7 @@ if os.environ.get('NZBPO_STATISTICS') == 'yes':
 
 # add list of downloaded files
 files = False
-if os.environ.get('NZBPO_FILELIST') == 'yes':
+if os.environ.get('NZBPO_FILELIST') == 'yes' and not test_mode:
 	text += '\n\nFiles:'
 	for dirname, dirnames, filenames in os.walk(os.environ['NZBPP_DIRECTORY']):
 		for filename in filenames:
@@ -216,18 +231,19 @@ if os.environ.get('NZBPO_FILELIST') == 'yes':
 		text += '\n<no files found in the destination directory (moved by a script?)>'
 
 # add _brokenlog.txt (if exists)
-if os.environ.get('NZBPO_BROKENLOG') == 'yes':
+if os.environ.get('NZBPO_BROKENLOG') == 'yes' and not test_mode:
 	brokenlog = '%s/_brokenlog.txt' % os.environ['NZBPP_DIRECTORY']
 	if os.path.exists(brokenlog):
 		text += '\n\nBrokenlog:\n' + open(brokenlog, 'r').read().strip()
 
 # add post-processing log
-if os.environ.get('NZBPO_NZBLOG') == 'Always' or \
-	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success):
+if (os.environ.get('NZBPO_NZBLOG') == 'Always' or \
+	(os.environ.get('NZBPO_NZBLOG') == 'OnFailure' and not success)) and \
+	not test_mode:
 
 	# To get the item log we connect to NZBGet via XML-RPC and call
 	# method "loadlog", which returns the log for a given nzb item.
-	# For more info visit http://nzbget.net/RPC_API_reference
+	# For more info visit http://nzbget.net/api
 
 	# Call remote method 'loadlog'
 	nzbid = int(os.environ['NZBPP_NZBID'])
@@ -240,8 +256,9 @@ if os.environ.get('NZBPO_NZBLOG') == 'Always' or \
 			text += '\n%s\t%s\t%s' % (entry['Kind'], datetime.datetime.fromtimestamp(int(entry['Time'])), entry['Text'])
 
 # Create message
-msg = MIMEText(text)
-msg['Subject'] = subject
+print('[DETAIL] Creating Email')
+msg = MIMEText(text.encode('utf-8'), 'plain', 'utf-8')
+msg['Subject'] = Header(subject, 'utf-8')
 msg['From'] = os.environ['NZBPO_FROM']
 msg['To'] = os.environ['NZBPO_TO']
 msg['Date'] = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
